global logFileFid

proc captureTimeConfiguration { par1 captureTime } {

	# capture configuration time
	$par1 paragonset Capture Control Mode FIXED
	$par1 paragonset Capture Control FixedPeriod USER
	$par1 paragonset Capture Control UserPeriod $captureTime

}



proc syncEConfiguration { par1 } {

	if { [ $par1 paragonget Capture SyncE WanderCaptEnable ] == TRUE } {
		# select packet generation with SSM code of QL_PRC
		$par1 paragonset PacketGeneration #0 Esmc #0 SsmType QL-PRC

	} else {
		# select packet generation with SSM code of QL_DNU
		$par1 paragonset PacketGeneration #0 Esmc #0 SsmType QL-DNU/DUS

	}
}



proc openCat {par1 measure} {

	# opening the Cat - fetching the results

    if { $measure == "PTP" } { 
		$par1 paragonset Cat Show TRUE
		$par1 paragonset Cat 1588TimeError
		$par1 waitForCat
    } elseif { $measure == "1PPS" } {
		$par1 paragonset Cat Show TRUE
		$par1 paragonset Cat 1PPS
		$par1 waitForCat
    } else {
		puts "INVALID MEASURE SELECTED"
    }
}

proc startMSE { par1 captureTime } {
	$par1 paragonset MasterSlave Master #0 Enabled TRUE

}

#
# Start MSE and wait deviceSettlingTime seconds.
# Set deviceSettlingTime to zero if no settling is required
# If captureFilename is not "", then take a capture and save the capture to the specified file
# captureFilename should be the complete path to the file and include the extension


#ENT-41 Need to pass in noiseTransferType to decide if Port2 needs checked
#
proc startMSEandWaitonSettling { par1 noiseTransferType  {deviceSettlingTime 0} {captureFilename ""} {isDemoMode FALSE}  } {

#  For some reason these need set before MSE runs
#
#
    par1 paragonset MasterSlave FlowFilter ImpairDelReq FALSE
    par1 paragonset MasterSlave FlowFilter ImpairMulticastAnnounce TRUE
    par1 paragonset MasterSlave FlowFilter ImpairMulticastSync TRUE
    par1 paragonset MasterSlave FlowFilter ImpairMulticastDelay TRUE
    par1 paragonset MasterSlave FlowFilter ImpairMulticastAllSlaves TRUE
    par1 paragonset MasterSlave FlowFilter ImpairSet TRUE
   
    XXXDebugTrace "Starting MSE..."
    $par1 paragonset MasterSlave Master #0 Enabled TRUE

	# Check that we have a 1pps reference
	if {[is1ppsReferenceDetected] == FALSE} {
		log "FATAL ERROR: No 1pps reference has been detected."
		exit
	}
	XXXDebugTrace "1pps reference is good..."
	
# AE - 41 need to change above to use noiseTransferType
    # Check if traffic is seen
    # If running a T-BC test need to check both links for packets, if testing a T-TSC need only check port1 for traffic
    
    set meas [par1 getMeasurementConfiguration]   
    
    if { $noiseTransferType == "PTP-1PPS" || $noiseTransferType == "SyncE-1PPS" } {
		if {[$meas portPacketsReceived 0] == FALSE} {
			log "FATAL ERROR: No packets are being received on Port 0. Please check your cabling and device configuration"
			exit
		    }
				XXXDebugTrace "Good packets being received on Port 1.."
	} else {
        if {[$meas portPacketsReceived 0] == FALSE || [$meas portPacketsReceived 1] == FALSE} {
		log "FATAL ERROR: No packets are being received on Port 1 and/or Port 2. Please check your cabling and device configuration"
            exit
        }
			XXXDebugTrace "Good packets being received on both ports..."
	}

	# If captureFilename is specified, start a capture
	# This will capture for the specified duration i.e. the settling time
	if {$captureFilename != ""} {
	    XXXDebugTrace "Capturing / waiting for device to settle - $deviceSettlingTime seconds"
		packetCapture $par1 $deviceSettlingTime $isDemoMode $captureFilename
	} else {	
	    # wait on device settling before starting impairment
	    XXXDebugTrace "Waiting for device to settle - $deviceSettlingTime seconds"
	    after [ expr {$deviceSettlingTime * 1000} ]  
		
	}

}

proc is1ppsReferenceDetected {} {
	set status FALSE
	# Give it 10 seconds to appear
	for {set i 0} {$i<10} {incr i} {
		set status [par1 paragonget InstrumentStatus Interface 1ppsRefLock Detected]
		if {$status == TRUE} { break }
		after 1000
	}
	return $status
}
proc is1ppsMeasurementInputDetected {} {
	set status FALSE
	# Give it 10 seconds to appear
	for {set i 0} {$i<10} {incr i} {
		set status [par1 paragonget InstrumentStatus Interface 1ppsMeasLock Detected]
		if {$status == TRUE} { break }
		after 1000
	}
	return $status
}

proc stopMSE { par1 captureTime } {
	$par1 paragonset MasterSlave Master #0 Enabled FALSE

}



proc packetCapture { par1 captureTime {isDemoMode FALSE} {filename ""}} {

	if {!$isDemoMode} {
	    captureTimeConfiguration par1 $captureTime
		$par1 paragonset MasterSlave StartMeasurement TRUE
	} else {
		log "Demo mode: calling starttimingcapture NOT StartMeasurement"
		$par1 starttimingcapture
	}
	
	after [ expr {$captureTime * 1000} ]
	
	if {!$isDemoMode} {

	    set downLoadActiveCount 240

		while { [$par1 paragonget Capture Active] == TRUE  && $downLoadActiveCount > 0 } {
			after 1000
		        incr downLoadActiveCount -1
				set dac [format "%3d" $downLoadActiveCount]
		    	XXXDebugTrace "[currentDateTime]\ Download Active count:\t$dac\r" FALSE
		}
		# Force a newline
		XXXDebugTrace ""
		set res [$par1 paragonget Capture Active]
		XXXDebugTrace "[currentDateTime]\tAfter stop loop Capture status:\t$res Download Active Count\t:$downLoadActiveCount"
	}
	
	# Save the capture to the specified file
	if {$filename != ""} {
		set filepath "[pwd]/$filename"
		XXXDebugTrace "Exporting capture to $filepath"
		if { [catch {par1 exportdata $filepath}] } {
			log "Failed to export capture file $filename (capture status: [par1 paragonget InstrumentStatus Capture IsRunning]). This is not fatal - continuing execution."
		}
		
	}
	
#	XXXDebugTrace "packetCapture: Stopping MSE..."
#	$par1 paragonset MasterSlave Master #0 Enabled FALSE
}

proc ObjectCleanUp {} {
	variable logFileFid
	
	ParagonX destroy
	Socket destroy
	Result destroy
	MeasurementAnalysis destroy
	Measurement destroy
	Port destroy
	References destroy
	MasterSlaveSetUp destroy
	close $logFileFid
}

proc openLogFile {folder file} {
	variable logFileFid
	
	if {![file exists $folder]} {
		file mkdir $folder
	}
	set logFileFid [open $file "w"]
}

proc log {text} {
	variable logFileFid
	
	puts $text
	puts $logFileFid $text
    flush $logFileFid
}

proc getCallerFunctionName {infoString} {
	set caller "UNDEFINED"
	# If the caller is in an object, then the format will be something like:
	#   ::oo::Obj<num>::objId functionName
	# e.g. ::oo::Obj44::step0 startStimulus <params>
	# If the caller isn't in an object then we just get the function name and params
	# So, if infoString doesn't start with "::oo", then take the first item in the list as the caller name
	# If infoString starts with "::oo", then extract the ObjId and the function name
	if {[string first "::oo::" $infoString] == 0} {
		# Caller is in an object
		set parts [split $infoString ":"]
		set remainder [lindex $parts 6]
		set remainingParts [split $remainder " "]
		set objId [lindex $remainingParts 0]
		set callerFunction [lindex $remainingParts 1]
		set caller "$objId.$callerFunction"
	} else {
		# Caller name is first in the list
		set caller [lindex $infoString 0]
	}
	return $caller
}

proc logLevel {level text} {
	#set callerFunctionName [lindex [info level -1] 0]
	set callerFunctionName [getCallerFunctionName [info level -1]]
	set l "!!!! $callerFunctionName ($level) !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
	log ""
	log $l
	log $text

	if {$level == "FATAL"} {
		log "Aborting..."
		exit
	}
}
# Use an optional parameter for newline or nonewline

proc  XXXDebugTrace { text {nl TRUE}} {
	global debugTrace
	variable logFileFid
	
	#set callerFunctionName [lindex [info level -1] 0]
	#set callerFunctionName [info level -1]
	set callerFunctionName [getCallerFunctionName [info level -1]]
	set t "********** DEBUG $callerFunctionName: $text"
	if {$nl == TRUE} {puts $logFileFid $t} else {puts -nonewline $logFileFid $t}
    if { $debugTrace } {
		if {$nl == TRUE} {puts $t} else {puts -nonewline $t}
	}
    flush $logFileFid
}

# Get the current time
proc currentDateTime {} {
	set systemTime [clock seconds]
	set d [clock format $systemTime -format "%Y-%m-%d"]
	set t [clock format $systemTime -format %H-%M-%S]
	return "$d $t"
}

# Get the minimum value in the supplied arguments
proc getMin args {
    if {[llength $args] == 1} {set args [lindex $args 0]}
    lindex [lsort -real $args] 0
}

# get the maximum value in the supplied arguments
proc getMax args {
    if {[llength $args] == 1} {set args [lindex $args 0]}
    lindex [lsort -real $args] end
}

