
oo::class create Measurement {
    variable paragonObject port1 port2 references master slave
    constructor { paragonObject1 } {
		set par1 $paragonObject1
		
		# Selecting Operating mode, 1588v2, Enable Master / Slave Emulation
		operatingModeMasterSlaveEmulation par1
		
		# Choosing test Configuration
        
		par1 paragonset MasterSlave TestConfiguration BOUNDARY_CLOCK
		
        # DJWAT -AE-7
        # Always run as uncoupled, just set ports up with the same values if coupled is selected
        par1 paragonset Physical Coupled FALSE
        
		set port1 [ Port new $par1 [self] 0]
		set port2 [ Port new $par1 [self] 1]
		set references [ References new $par1 [self] ]
		set master [ Master new $par1 [self] ]
		set slave [ Slave new $par1 [self] ]
    }
    
    destructor {      
    }
	
    method measurementConfiguration { pdhWander onePps syncE } {
		# Measurements settings 


		try {	
		    # Catch situation whereby customers do not have Opton 205 (Wander Measurements) enabled
		    if { $pdhWander == "None" } {
				par1 paragonset Capture Pdh T1WanderCaptEnable FALSE
				par1 paragonset Capture Pdh E1WanderCaptEnable FALSE
				par1 paragonset Capture Pdh M2WanderCaptEnable FALSE
		    } else {
				par1 paragonset Capture Pdh $pdhWander TRUE
		    }
		} on error { information detailedInformation } {
			log $information
			#log $detailedInformation
		}

		
		if { $onePps == "None" } {
			par1 paragonset Capture OnePps AccuracyCaptEnable FALSE
			par1 paragonset Capture OnePps WanderCaptEnable FALSE
		} else {
			par1 paragonset Capture OnePps $onePps TRUE
		}
		
		# SyncE connection
		par1 paragonset Capture SyncE WanderCaptEnable $syncE
		
    }
	
    #DJWAT AE-7
    method portSetLineRate { lineRate portNumber } {
        # DJWAT AE-7
        if {$portNumber == 1} {
            $port2 setLineRate $lineRate
        } else {
            $port1 setLineRate $lineRate
        }
        
    }
	
	method portSetInterface { interface portNumber} {
        # DJWAT AE-7
        if {$portNumber == 1} {
    		$port2 setInterface $interface
        } else {
    		$port1 setInterface $interface
        }
    }
	
	method portLinkIsUp { portNumber } {
		set status FALSE
		if {$portNumber == 1} {set port $port2} else {set port $port1}
		# Check the link status. Try for 10 seconds...
		for { set i 0 } {$i < 10} {incr i} {
			set status [$port getLinkStatus]
			if {$status == TRUE} {break}
			after 1000
		}
		return $status
	}

	method portPacketsReceived { portNumber } {
		set status FALSE
		if {$portNumber == 1} {set port $port2} else {set port $port1}
		# Check the packet status. Try for 10 seconds...
		for { set i 0 } {1 < 10} {incr i} {
			set status [$port getPacketStatus]
			if {$status == TRUE} {break}
			after 1000
		}
		return $status
	}
	
	method referencesSetClockSource { clock } {
		$references setClockSource $clock
    }
	
	method masterSetEncapsulation { encapsulation } { 
		$master setEncapsulation $encapsulation
	}
	
	method masterConfigureMulticast { } {
		$master configureMulticast
	}
	
	method masterSetMessagesFrequencies { announce sync } {
		$master setMessagesFrequencies $announce $sync
	}
	
	method slaveSetEncapsulation { encapsulation } { 
		$slave setEncapsulation $encapsulation
	}
	
	method slaveConfigureMulticast { } {
		$slave configureMulticast
	}
	
	method slaveSetAllowedMasters { delReqRate } {
		$slave setAllowedMasters $delReqRate
	}
	
	method setCableDelay { cableDelay } { 
		# configure DUT to Paragon-x Cable Delay
		par1 paragonset MasterSlave BoundaryClockCalibration $cableDelay
	}
	
}

# =================================================================================================

oo::class create Port {
    variable par1 measurement
	variable portId
    
    constructor { paragonObject1 measurement1 portNumber} {
		set par1 $paragonObject1
		set measurement $measurement1
		set portId "#$portNumber"
	}
	
    destructor {      
    }
    
    method setLineRate { lineRate } {
		# Configuring Line Rate in Setup Interface
	
    	# DJWAT AE-7
        par1 paragonset Physical $portId LineRate $lineRate
    	#par1 paragonset Physical LineRate $lineRate
		
		# par1 paragonset Physical LineRate 1GBE
		# par1 paragonset Physical LineRate 100BASET
		# par1 paragonset Physical LineRate 10GBE
    }
	
	method setInterface { interface } {
	
	
        # DJWAT-AE-7        
    	if { $interface == "OPTICAL"} {
			par1 paragonset Physical $portId LineInterface $interface
		}
		if { $interface == "ELECTRICAL" } {
			par1 paragonset Physical $portId LineInterface $interface
		} 
		if { $interface == "XFP" } {
			par1 paragonset Physical $portId LineRate 10GBE
			par1 paragonset Physical $portId xFPSelect XFP
		}
		if { $interface == "SFPPLUS" } {
			par1 paragonset Physical $portId LineRate 10GBE
			par1 paragonset Physical $portId xFPSelect SFPPLUS
		}
    	#if { $interface == "OPTICAL"} {
		#	par1 paragonset Physical LineInterface $interface
		#}
		#if { $interface == "ELECTRICAL" } {
		#	par1 paragonset Physical LineInterface $interface
		#} 
		#if { $interface == "XFP" } {
		#	par1 paragonset Physical LineRate 10GBE
		#	par1 paragonset Physical xFPSelect XFP
		#}
		#if { $interface == "SFPPLUS" } {
		#	par1 paragonset Physical LineRate 10GBE
		#	par1 paragonset Physical xFPSelect SFPPLUS
		#}
		# par1 paragonset Physical xFPSelect XFP
		# par1 paragonset Physical xFPSelect SFPPLUS
		# par1 paragonset Physical LineInterface ELECTRICAL
    }

	method getLinkStatus {} {
		return [par1 paragonget InstrumentStatus Interface $portId Link Detected]
	}
	method getPacketStatus {} {
		return [par1 paragonget InstrumentStatus Interface $portId RxPackets Good]
	}
}

# =================================================================================================

oo::class create References {
    variable par1 measurement
    
    constructor { paragonObject1 measurement1 } {
		set par1 $paragonObject1
		set measurement $measurement1
    }
	
    destructor {      
    }
    
    method setClockSource { clock } {
		# Configure stable reference for Paragon-x, setting the clock
		par1 paragonset Physical RefClkSource $clock
		
		# par1 paragonset Physical RefClkSource EXT_10M
		# par1 paragonset Physical RefClkSource EXT_R75_E1
		# par1 paragonset Physical RefClkSource EXT_R100_T1
		# par1 paragonset Physical RefClkSource EXT_R100_E1
		# par1 paragonset Physical RefClkSource EXT_K64
		# par1 paragonset Physical RefClkSource INT
    }
}

# =================================================================================================

oo::class create MasterSlaveSetUp {
    variable par1 measurement
    
    constructor { paragonObject1 measurement1 } {
		set par1 $paragonObject1
		set measurement $measurement1
    }
	
    destructor {      
    }
	
}


# =================================================================================================

oo::class create Master {
    variable par1 measurement
	superclass MasterSlaveSetUp
    
    constructor { paragonObject1 measurement1 } {
		next $paragonObject1 $measurement1
    }
	
    destructor {      
    }
	
	method setEncapsulation { encapsulation } { 
		$par1 paragonset MasterSlave Master #0 Encapsulation $encapsulation
	}
	
	method configureMulticast { } {
		$par1 paragonset MasterSlave Master #0 MulticastEnabled TRUE
		$par1 paragonset MasterSlave Master #0 EnableMcastAnnMsgs TRUE
		$par1 paragonset MasterSlave Master #0 MulticastSyncEnabled TRUE
		$par1 paragonset MasterSlave Master #0 MulticastDelRespEnabled TRUE
#		$par1 paragonset MasterSlave Master #0 UnicastEnabled FALSE
	}
	
	method setMessagesFrequencies { announce sync } {
		$par1 paragonset MasterSlave Master #0 MulticastAnnRate $announce
		$par1 paragonset MasterSlave Master #0 MulticastSyncRate $sync
	}
}

# =================================================================================================

oo::class create Slave {
    variable par1 measurement
	superclass MasterSlaveSetUp
    
    constructor { paragonObject1 measurement1 } {
		next $paragonObject1 $measurement1
    }
	
    destructor {      
    }
	
	method setEncapsulation { encapsulation } { 
		$par1 paragonset MasterSlave Slave Encapsulation $encapsulation
	}
	
	method configureMulticast { } {
#		$par1 paragonset MasterSlave Slave UnicastAnnounce FALSE
#		$par1 paragonset MasterSlave Slave UnicastSync FALSE
#		$par1 paragonset MasterSlave Slave UnicastDelResp FALSE
	    	$par1 paragonset MasterSlave Slave AutoDiscoverMACMulticastMaster TRUE

	}
	
	method setAllowedMasters { delReqRate } {
		$par1 paragonset MasterSlave Slave DelRespMsgRate $delReqRate
	}
	
}

# =================================================================================================

proc operatingModeMasterSlaveEmulation { par1 } {

	$par1 paragonset SimulMeasImpairMode MEASUREONLY
	$par1 paragonset OperatingMode PTP
	$par1 paragonset MasterSlave Enabled TRUE
	
}
