# ########################################################################################################
# NoiseGeneration.tcl
# Created: 6th July 2018
# Author: GM
# --------------------------------------------------------------------------------------------------------
# Measures the time error output from the DUT in the absence of any noise on the input
# --------------------------------------------------------------------------------------------------------
# Parameters:
#	par1: An instance of a paragon object
# Return values:
#
# Pre-Conditions:
# The instrument is already configured for G.8273.2 noise generation measurements
# MSE is enabled and has been running long enough to allow the DUT to settle
# --------------------------------------------------------------------------------------------------------
source "./Common Functions.tcl"

proc noiseGenerationInitialise {} {
	
}


proc noiseGenerationDoCapture {par1 captureDuration captureFolder captureFile demo} {
	#set captureFile "noiseGeneration.cpd"
	XXXDebugTrace "Starting packet capture: $captureDuration seconds"

	if {!$demo} {
		packetCapture par1 $captureDuration $demo
	}

	# explicitly stop capture, possible differences in GUI behavious
	# just in case catch, should an older version throw an error on stopping an already stopped capture
	XXXDebugTrace "Stopping packet capture..."
	catch {
		par1 stopcapture
#		puts "capture stopped"
	}

	if {!$demo} {
		set f ""
		#append f [pwd] "/$folder/" $captureFile
		append f "$captureFolder/" $captureFile
		XXXDebugTrace "Exporting capture to $f"
		if { [catch {par1 exportdata $f}] } {
			logLevel "WARNING" "Failed to export capture file $captureFile (capture status: [par1 paragonget InstrumentStatus Capture IsRunning]). This is not fatal - continuing execution."
		}
	}

}

proc noiseGenerationGetStats {TEType resultObject} {
	set resultsDict [dict create]
	
	if {$TEType == "2Way"} {
		dict set resultsDict "range" [ $resultObject getMax_Min ]
		dict set resultsDict "max" [ $resultObject getMax ]
		dict set resultsDict "min" [ $resultObject getMin ]
		dict set resultsDict "mean" [ $resultObject getMean ]
		dict set resultsDict "fwdMessages" [ $resultObject getFwdMessages ]
		dict set resultsDict "revMessages" [ $resultObject getRevMessages ]
		dict set resultsDict "fwdRate" [ $resultObject getForwardRate ]
		dict set resultsDict "revRate" [ $resultObject getReverseRate ]
	} elseif {$TEType == "T1"} {
		dict set resultsDict "range" [ $resultObject getMax_Min ]
		dict set resultsDict "max" [ $resultObject getMax ]
		dict set resultsDict "min" [ $resultObject getMin ]
		dict set resultsDict "mean" [ $resultObject getMean ]
		dict set resultsDict "fwdMessages" [ $resultObject getMessages ]
		dict set resultsDict "revMessages" NA
		dict set resultsDict "fwdRate" [ $resultObject getRate ]
		dict set resultsDict "revRate" NA		
	} elseif {$TEType == "T4"} {
		dict set resultsDict "range" [ $resultObject getMax_Min ]
		dict set resultsDict "max" [ $resultObject getMax ]
		dict set resultsDict "min" [ $resultObject getMin ]
		dict set resultsDict "mean" [ $resultObject getMean ]
		dict set resultsDict "fwdMessages" NA
		dict set resultsDict "revMessages" [ $resultObject getMessages ]
		dict set resultsDict "fwdRate" NA
		dict set resultsDict "revRate" [ $resultObject getRate ]		
	} elseif {$TEType == "1PPS"} {
		dict set resultsDict "range" [ $resultObject getMax_Min ]
		dict set resultsDict "max" [ $resultObject getMax ]
		dict set resultsDict "min" [ $resultObject getMin ]
		dict set resultsDict "mean" [ $resultObject getMean ]
	} else {
		logLevel "FATAL" "TEType $TEType unrecognised"
		#exit
	}
}

proc noiseGenerationGetResults {ptpOr1pps TEType statistic} {		
	# addMeasurement will fail if the CAT cannot select the appropriate slot
	# Catch any fail, log an error message and terminate
	if {[catch {set meas [par1 addMeasurement "$ptpOr1pps.$TEType.$statistic" "TE" $TEType]}]} {
		logLevel "FATAL" "There are no $ptpOr1pps ($TEType) results\n\
		Please make sure that you can successfully make time error measurements manually before running this script"
		#log "noiseGenerationGetResults (FATAL): There are no $ptpOr1pps ($TEType) results"
		#log "Please make sure that you can successfully make time error measurements manually before running this script"
	}
	set measResult [ $meas createResult ]
	set allResults [noiseGenerationGetStats $TEType $measResult]
	set result [dict get $allResults $statistic]
	$measResult destroy
	$meas destroy
	return $result
}

proc noiseGeneration {par1 captureFolder captureFilename argsDict demo reProcessFolder} {
	set captureDuration [dict get $argsDict "captureDuration"]
	noiseGenerationDoCapture par1 $captureDuration $captureFolder $captureFilename $demo

	if {$demo} {
		XXXDebugTrace "Demo mode - Importing $reProcessFolder/$captureFilename"	
		par1 importdata "$reProcessFolder/$captureFilename"	
	}
	
	set measurements [dict get $argsDict "measurements"]
	# Measurements is a dictionary with keys of either "PTP" or "1pps"
	# The value associaed with the key is a list of T1/T4/2Way followed by the statistic
	# For 1pps, the list is "1pps" followed by the statistic
	foreach {ptpOr1pps measList} $measurements {
		XXXDebugTrace "Opening CAT ($ptpOr1pps)..."
		openCat par1 $ptpOr1pps
		#puts "$ptpOr1pps: $measList"
		foreach {m s} $measList {
			set result [noiseGenerationGetResults $ptpOr1pps $m $s]
			#puts "$m: $s=$result"
			dict set resultsDict "$m$s" $result
		}
	}
	return $resultsDict
}

