# // //////////////////////////////////////////////////
# Wrapper functions for direct use of restful api
package require rest
# // //////////////////////////////////////////////////
proc _req {req url body} {
	set httpErrorDict [dict create 400 "Bad Request" 404 "Not Found" 500 "Internal Server Error" 501 "Not Implemented"]
	global [namespace current]::__instrument
	set resp ""
	set config "method ${req} headers \{Content-Type application/json\}"
	#return [rest::simple "${__instrument}${url}?format=json" "" $config $body] 
	
	if { [catch {
		set resp [rest::simple "${__instrument}${url}?format=json" "" $config $body]
	} errmsg ] } {
		set httpErrorString ""
		set httpErrorNum [lindex [split $errmsg] 1]
		if {[dict exists $httpErrorDict $httpErrorNum]} {
			set httpErrorString [dict get $httpErrorDict $httpErrorNum]
		}
		throw $httpErrorNum "HTTP $httpErrorNum: $httpErrorString ($url)"
	}
	return $resp
}
proc _get {url args} {
	set res [json::json2dict [_req get $url ""]]
	if {[llength $args] == 0} {
		return $res
	} else {
		return [dict get $res {*}$args]
	}
}

proc _getCatStatus {} {
	set resp [_get "/cat/general/status"]
	return $resp
}

proc _getCatExportFolder {} {
	set resp [_get "cat/filecommander/ExportFolder"]
	return $resp
}

if { $argc == 0 } {
	puts "Please specify the IP address of the PC on which the CAT is running"
} else {
	set ipAddress $argv
}

puts "Testing communication with CAT on PC $ipAddress"

set __instrument "http://$ipAddress:1337/api/"

if {[catch {set resultDict [_getCatStatus]}]} {
    puts stderr "Unable to communicate with CAT. Please make sure that the CAT is running on $ipAddress"
    exit 1
}

puts "CAT Version: [dict get $resultDict "VersionNumber"]"
puts "CAT Application Name: [dict get $resultDict "ApplicationName"]"
puts "CAT Service Server Address: [dict get $resultDict "ServiceServerAddress"]"
puts "CAT XSocket Server Address: [dict get $resultDict "XsocketServerAddress"]"
puts ""

set resultDict [_getCatExportFolder]
puts "CAT Export Folder Exists: [dict get $resultDict "IsDirectory"]"
set folderContents [dict get $resultDict "ChildrenElements"]
puts "Folder Contents:"
foreach i $folderContents {
	puts "\t$i"
}


