# script file for noise transfer in TCL
# Note:
# This function assumes a sample rate of 16 pkts/sec.
# It is likely that for real devices, the sample rate will not be exactly 16.
# The fix is probably to calculate the sample rate across the capture or make use of the timestamps directly.
# Not sure if the latter is possible.

proc filterIIR { stimulus testFile results resultsFile inputFreq inputAmpl inputSampleRate } { 
	variable pi

	XXXDebugTrace "Using IIR filter... $inputFreq $inputAmpl $inputSampleRate"

	# open file to write
    set fileId [ open $resultsFile "a"]

	append intro "t1" "," "d1" "," "filtered_data_q" "," "filtered_data2_q" "," "filtered_data3_q" "," "filtered_data_i" "," "filtered_data2_i" "," "filtered_data3_i" "," 
	puts $fileId $intro

	#set pi 3.1415926535897931

	set inputAmplitude $inputAmpl 
	set inputFrequency $inputFreq
	set Sample_Rate_Hz $inputSampleRate 

	#puts $results
	set d1 [lindex $results 1]
	#set d1 $results

	XXXDebugTrace "filterIIR: Initialising..."
	set Wc [expr {2*$pi*$inputFrequency} ]

	# Data is now coming in as a list (not an array)
	# Converting a list to array appears to be non-trivial
	# It looks as though initialising I and Q to the data values should not be required so skipping it - if anything looks odd though, this would be the palce to look
	#array set Q1 [array get d1]
	#array set I1 [array get d1]
	
	set Low_Pass_Frequency_Hz 0.01
	# 3dB radian frequency
	set W3 [expr {[ expr {$Low_Pass_Frequency_Hz / $Sample_Rate_Hz} ] * 2 * $pi}]   
	# Scaled 3dB radian frequency  
	set Ohmega3 [expr {tan([expr $W3/2])} ]

	# Compute the digital filter coefficients
	set A0 [expr {$Ohmega3 / (1.0 + $Ohmega3)}]
	set A1 $A0
	set B1 [expr {-($Ohmega3 - 1.0)/(1.0+ $Ohmega3)}]

	# Set up temporary variables to hold filter data values
	set previous_input_i 0
	set previous_output_i 0

	# Temp variables for 2nd stage filter I values
	set previous_input2_i 0
	set previous_output2_i 0

	# Temp variables for 3rd stage filter values
	set previous_input3_i 0
	set previous_output3_i 0


	set previous_input_q 0
	set previous_output_q 0

	# Temp variables for 2nd stage filter Q values
	set previous_input2_q 0
	set previous_output2_q 0

	# Temp variables for 3nd stage filter Q values
	set previous_input3_q 0
	set previous_output3_q 0

	# Variables to hold the sum of I and Q
	set I_SUM 0.0
	set Q_SUM 0.0

	# No of samples in the average
	set SUM_COUNT 0

	XXXDebugTrace "filterIIR: Filtering... [llength $d1] samples"
	set n [expr {[llength $d1] - 1}]
	#for { set i 0}  {$i < [array size d1]} {incr i} 
	for { set i 0}  {$i < $n} {incr i} {

		# Now do the I & Q mixing  
		##set I1($i) [ expr [ expr $d1($i)] *cos($Wc*$i*1/$Sample_Rate_Hz) ] 
		##set Q1($i) [ expr [ expr $d1($i)] *sin($Wc*$i*1/$Sample_Rate_Hz) ]
		set d [lindex $d1 $i]
		#puts $d
		set ival [ expr {$d*cos($Wc*$i*1/$Sample_Rate_Hz)} ] 
		set qval [ expr {$d*sin($Wc*$i*1/$Sample_Rate_Hz)} ]
		#puts "$ival $qval"
		set I1($i) $ival
		set Q1($i) $qval

		# Now filter the mixing products to remove the double frequency component

		set filtered_data_i($i) [ expr {$A0 * $I1($i) + $A1*$previous_input_i + $B1*$previous_output_i} ]
		set previous_input_i $I1($i)
		set previous_output_i $filtered_data_i($i)

		# Now apply filter 2nd time on I channel

		set filtered_data2_i($i) [ expr {$A0*$filtered_data_i($i) + $A1*$previous_input2_i + $B1*$previous_output2_i} ]
		set previous_input2_i $filtered_data_i($i)
		set previous_output2_i $filtered_data2_i($i)

		# Now apply filter 3rd time on I channel

		set filtered_data3_i($i) [ expr {$A0*$filtered_data2_i($i) + $A1*$previous_input3_i + $B1*$previous_output3_i} ]
		set previous_input3_i $filtered_data2_i($i)
		set previous_output3_i $filtered_data3_i($i)

		# Filter the Q channel

		set filtered_data_q($i) [ expr {$A0*$Q1($i) + $A1*$previous_input_q + $B1*$previous_output_q} ]
		set previous_input_q $Q1($i)
		set previous_output_q $filtered_data_q($i)

		# Now filter the Q channel a 2nd time

		set filtered_data2_q($i) [ expr {$A0*$filtered_data_q($i) + $A1*$previous_input2_q + $B1*$previous_output2_q} ]
		set previous_input2_q $filtered_data_q($i)
		set previous_output2_q $filtered_data2_q($i)

		# Now filter the Q channel a 3rd time

		set filtered_data3_q($i) [ expr {$A0*$filtered_data2_q($i) + $A1*$previous_input3_q + $B1*$previous_output3_q} ]
		set previous_input3_q $filtered_data2_q($i)
		set previous_output3_q $filtered_data3_q($i)

		# Now form the sum of I**2 and Q**2 and record the no of samples in the sum	
	
		# Performs the sum on the last 1/3 of the data
		#if {$i > [ expr 2.0 * [ array size d1 ] / 3 ] } 
		if {$i > [ expr 2.0 * [ llength $d1 ] / 3 ] } {
			set I_SUM [ expr {$I_SUM + $filtered_data3_i($i)} ] 
			set Q_SUM [ expr {$Q_SUM + $filtered_data3_q($i)} ]
			incr SUM_COUNT
		}
		set info "" 
		#append info $i "," $d1($i) "," $filtered_data_q($i) "," $filtered_data2_q($i) "," $filtered_data3_q($i) "," $filtered_data_i($i) "," $filtered_data2_i($i) "," $filtered_data3_i($i) "," 
		append info $i "," $d "," $filtered_data_q($i) "," $filtered_data2_q($i) "," $filtered_data3_q($i) "," $filtered_data_i($i) "," $filtered_data2_i($i) "," $filtered_data3_i($i) "," 
		# puts $info
		puts $fileId $info
	}

	# Amplitude (RMS) of sine wave (note multiply by 2 is to compensate for the initial mixing which halves the amplitude)

	set AMPLITUDE [expr {2.0*sqrt( [ expr $I_SUM*$I_SUM + $Q_SUM*$Q_SUM] / [ expr $SUM_COUNT*$SUM_COUNT] )} ]

	# return pk-pk value, therefore double the initial amplitude
	set AMPLITUDE [ expr {2* $AMPLITUDE}]

	#puts "IIR: Input Amplitude: $inputAmplitude, Output Amplitude:  $AMPLITUDE "
	close $fileId

	set resultList [ list $inputAmplitude $AMPLITUDE $inputFrequency ]
	return $resultList
}




