source "./paragonwl.tcl"
source "./results.tcl"



oo::class create MeasurementAnalysis {
    variable result paragonObject timeMeasurement
    
    constructor { paragonObject1 } {
        set paragonObject $paragonObject1
    }

    
    destructor {
        
    }
    
    method getResult {} {
        return $result
    }
    
    method createResult { measurementAnalysis timeMeasurement } {
 


       if { $timeMeasurement == "2Way" } {
            set result [ ResultSet1 new par1 $measurementAnalysis $timeMeasurement ]
        } elseif { $timeMeasurement == "1pps" } {

            set result [ ResultSet1 new par1 $measurementAnalysis $timeMeasurement ]
        } else {
            set result [ ResultSet2 new par1 $measurementAnalysis $timeMeasurement ]
        }
        return $result
    }
    
    method selectSlot { timeMeasurement } {
 
       $paragonObject paragonset Cat SelectSlot $timeMeasurement
        $paragonObject paragonset Cat EnabledSlot True
    }
    
    method enableMeasurement { measurementAnalysis } {
        $paragonObject paragonset Cat $measurementAnalysis Enable true
    }
    
}

# =================================================================================================


oo::class create TransparentClockAccuracyMeasurementAnalysis {
    superclass MeasurementAnalysis

    constructor { paragonObject1 } {
        next $paragonObject1 
    }
    
    destructor {      
    }
}

# =================================================================================================


oo::class create TimeErrorMeasurementAnalysis {
    variable paragonObject
    variable timeMeasurement limitEnable limitValue
    superclass MeasurementAnalysis
    
    constructor { paragonObject1 } {
        next $paragonObject1 
    }
    
    destructor {     
    }
    

    method errorCheckIncludeCorrectionField {} {
        if { $timeMeasurement == "1pps"} {
            error " Error: Include Correction Field configuration is not avaliable for $timeMeasurement "            
        }
    }
    
    
    method createResult { measurementAnalysis } {
        
        next $measurementAnalysis $timeMeasurement

    }   
    
    method getTimeMeasurement {} {

        return $timeMeasurement

    }
    
    method setLimitEnable { measurementAnalysis value } {
        my selectSlot $timeMeasurement
        $paragonObject paragonset Cat $measurementAnalysis ThresholdLimitEnabled $value
        set limitEnable $value
    }
    
    method getLimitEnable { measurementAnalysis } {
        my selectSlot $timeMeasurement
        if { [info exists limit] } {
            return $limitValue
        } else {
            set limitEnable [ $paragonObject paragonget Cat $measurementAnalysis ThresholdLimitEnabled ]
            return $limitEnable
        }
    }
    
    method setLimitValue { measurementAnalysis value } {
        my selectSlot $timeMeasurement
        $paragonObject paragonset Cat $measurementAnalysis ThresholdLimit $value
        set limitValue $value
    }
       
       
    method getLimitValue { measurementAnalysis } {
        my selectSlot $timeMeasurement
        if { [info exists limitValue] } {
            return $limitValue
        } else {
            set limitValue [ $paragonObject paragonget Cat $measurementAnalysis ThresholdLimit]
            return $limitValue
        }
    }
    
}

# =================================================================================================

oo::class create TETCMeasurementAnalysis {
    variable paragonObject
    variable timeMeasurement measurementAnalysis
    variable measurementAnalysisList
    superclass TransparentClockAccuracyMeasurementAnalysis

    
    constructor { paragonObject1 timeMeasurement1 } {
        next $paragonObject1
        set timeMeasurement $timeMeasurement1
    }
    
    destructor {      
    }
    
    
    method errorCheckMeasurementAnalysis {} {
        
        if { ![info exists measurementAnalysis]} {
            error " Error: Measurement analysis was not defined "
        }
    }
    
    
    method createResult {} {
        my errorCheckMeasurementAnalysis
        next $measurementAnalysis $timeMeasurement
    }
    
    
    method getTimeMeasurement {} {
        
        return $timeMeasurement
    }
    
    method setMeasurementAnalysis { measurementAnalysisType } {
        
        if { [lsearch $measurementAnalysisList $measurementAnalysisType] >= 0 } {
            
                if { $timeMeasurement == "Sync" } {
                    append measurementAnalysis "FWD_" $measurementAnalysisType
                    
                } elseif { $timeMeasurement == "Delay Req" } { 
                    append measurementAnalysis "REV_" $measurementAnalysisType
                    
                } elseif { $timeMeasurement == "2Way" && $measurementAnalysisType == "CF_ACCURACY" } {
                    append measurementAnalysis "TWOWAY_" $measurementAnalysisType
                } else {
                    error " Error: Incorrect measurement analysis: $timeMeasurement $measurementAnalysisType "
                }
                my enableMeasurement $measurementAnalysis
                        
        } else {
            error " Error: Incorrect measurement analysis: $measurementAnalysisType, possilbe measurement analysis are $measurementAnalysisList "
        }
        
    }
    
    method getMeasurementAnalysis {} {
        my errorCheckMeasurementAnalysis
        return $measurementAnalysis
    }
}

# =================================================================================================

oo::class create TEMeasurementAnalysis {
    variable paragonObject timeMeasurement
    superclass TimeErrorMeasurementAnalysis 
    
    constructor { paragonObject1 timeMeasurement1 } {
        next $paragonObject1
        set timeMeasurement $timeMeasurement1
        my enableMeasurement "TE"
 
   }
    
    destructor {      
    }
    
    method createResult { } {
        next "TE"
    }
    
    method errorCheckZero1pps {} {
        if { $timeMeasurement != "1pps" } {
            error " Error: Zero1pps configuration is avaliable only for 1pps measurement, not for $timeMeasurement "
        }
    }
    
    
    method setIncludeCorrectionField { value } {
        my selectSlot $timeMeasurement
        my errorCheckIncludeCorrectionField
        $paragonObject paragonset Cat TIMEERROR IncludeCorrectionField $value
        $paragonObject calculateRequiredSetTrue
    }
    
    method getIncludeCorrectionField {} {
        my selectSlot $timeMeasurement
        my errorCheckIncludeCorrectionField
        return [ $paragonObject paragonget Cat TIMEERROR IncludeCorrectionField ]
    }
    
    method setZero1pps { value } {
        my selectSlot $timeMeasurement
        my errorCheckZero1pps
        $paragonObject paragonset Cat TIMEERROR RemoveOffset $value
        $paragonObject calculateRequiredSetTrue ;#  not sure if needed
    }
    
    method getZero1pps {} {
        my selectSlot $timeMeasurement
        my errorCheckZero1pps
        $paragonObject paragonget Cat TIMEERROR RemoveOffset
    }
    
    
    method setLimitEnable { value } {
        next "TE" $value
    }
    
    method getLimitEnable {} {
        next "TE"
    }
    
    method setLimitValue { limitValue } {
        next "TE" $limitValue
    }
    
    method getLimitValue {} {
        next "TE"
    }
    
    #method setRange { bottomRangeValue topRangeValue } {
    #}
    #
    #method getRange {} {
    #}
}


# =================================================================================================

oo::class create ATEMeasurementAnalysis {
    variable paragonObject timeMeasurement    
    superclass TimeErrorMeasurementAnalysis    
    
    constructor { paragonObject1 timeMeasurement1 } {
        next $paragonObject1
        set timeMeasurement $timeMeasurement1
        my enableMeasurement "ATE"
    }
    
    destructor {      
    }
    
    method createResult { } {
        next "ATE"
    }
    
    
    method setIncludeCorrectionField { value } {
        my selectSlot $timeMeasurement
        my errorCheckIncludeCorrectionField
        $paragonObject paragonset Cat AVERAGEDTE IncludeCorrectionField $value
        $paragonObject calculateRequiredSetTrue

    }
    
    method getIncludeCorrectionField {} {
        my selectSlot $timeMeasurement
        my errorCheckIncludeCorrectionField
        return [ $paragonObject paragonget Cat AVERAGEDTE IncludeCorrectionField ]
    
    }
    
    method setAveragingPeriod { averagingPeriodValue } {
        my selectSlot $timeMeasurement
        $paragonObject paragonset Cat AVERAGEDTE AveragingPeriod $averagingPeriodValue
        $paragonObject calculateRequiredSetTrue
    }
    
    method getAveragingPeriod {} {
        return [ $paragonObject paragonget Cat AVERAGEDTE AveragingPeriod ]
    }
    
    method setLimitEnable { value } {
        next "ATE" $value
    }
    
    method getLimitEnable {} {
        next "ATE"
    }
    
    method setLimitValue { limitValue } {
        next "ATE" $limitValue
    }
    
    method getLimitValue {} {
        next "ATE"
    }
    
}

# =================================================================================================

oo::class create DTELF_HFMeasurementAnalysis {
    variable paragonObject timeMeasurement
    variable filterType
    superclass TimeErrorMeasurementAnalysis

    
    constructor { paragonObject1 timeMeasurement1 } {
        next $paragonObject1
        set timeMeasurement $timeMeasurement1
        #set filterTypesList { LF HF }
    }
    
    destructor {      
    }

    
    method errorCheckLimitConfigurationAvailability {} {
        
        if { $timeMeasurement != "2Way" && $timeMeasurement != "1pps" } {
            error " Error: setting Limit is not possible for $timeMeasurement measurement, only for 2Way and 1pps"
        }
    }
    
    method createResult { } {

        if { $filterType == "LF" } {
            set measurementAnalysis "DTE"
        } else {
            set measurementAnalysis "DTEHF"
        }
        next $measurementAnalysis
    }
    
    method setFilterType { type } {
        
        my selectSlot $timeMeasurement
        set filterType $type
        if { $filterType == "LF" } {
            my enableMeasurement "DTE"
        } elseif { $filterType == "HF" } {
            my enableMeasurement "DTEHF"
        } else {
            error " Error: Incorrect filter type: $type, Possible filter types are $filterTypesList "
        } 
    }
    
    method getFilterType {} {
        
        return $filterType
    }
    
    method setLimitEnable { value } {
        
        my errorCheckLimitConfigurationAvailability
        if { $filterType == "LF" } {
            next "DTE" $value
        } else {
            next "DTEHF" $value
        }
    }
    
    method getLimitEnable {} {
        
        my errorCheckLimitConfigurationAvailability
        if { $filterType == "LF" } {
            next "DTE"
        } else {
            next "DTEHF" 
        }
    }
    
    method setLimitValue { limitValue } {
        
        my errorCheckLimitConfigurationAvailability
        if { $filterType == "LF" } {
            next "DTE" $limitValue
        } else {
            next "DTEHF" $limitValue
        }
    }
    
    method getLimitValue {} {
        
        my errorCheckLimitConfigurationAvailability
        if { $filterType == "LF" } {
            next "DTE"
        } else {
            next "DTEHF" 
        }
    }   
}

# =================================================================================================

oo::class create DTEMTIELF_TDEVLFMeasurementAnalysis {
    variable paragonObject timeMeasurement
    variable masksListMTIE masksListTDEV
    superclass TimeErrorMeasurementAnalysis
    
    variable filterType
    
    constructor { paragonObject1 timeMeasurement1 } {
        next $paragonObject1
        set timeMeasurement $timeMeasurement1
        set masksListMTIE { "G.8261.1" "G.8263" "G.8271.1 Network Limit" "G.8273.2 dTE Gen Const Temp"\
        "G.8273.2 dTE Gen Var Temp" "GR.1244 Wander Generation" "G.811 Wander Generation" \
        "G.813 Wander Generation SEC Op1" "G.813 Wander Generation SEC Op1 +Temp" \
        "G.813 Wander Generation SEC Op2" "G.813 Wander Transient SEC Opt2" "G.823 E1 Traffic Interface 2048" \
        "G.823 E1 PDH Sync Interface" "G.823 E1 PRC Interface" "G.823 E1 SEC Interface" "G.823 E1 SSU Interface"\
        "G.824 Traffic Interface 1544" "G.824 T1 PRC Interface" "G.8261 Wander Limits EEC Op1"\
        "G.8262 Wander Generation EEC Op1" "G.8262 Wander Generation EEC Op1 +Temp"\
        "G.8262 Wander Generation EEC Op2" "G.8262 Wander Transient EEC Opt2" "G.8272 PRTC" }
        
        set masksListTDEV { "G.8273.2 dTE Gen Const Temp" "GR.1244 Wander Generation" "G.811 Wander Generation" \
        "G.813 Wander Generation SEC Op1" "G.813 Wander Generation SEC Op2" "G.813 Wander Transfer SEC Opt2"\
        "G.823 El PDH Sync Interface" "G.823 El PRC Interface" "G.823 El SEC Interface" "G.823 El SSU Interface"\
        "G.824 Traffic Interface 1544" "G.824 T1 PRC Interface" "G.8261 Wander Limits EEC Op1"\
        "G.8261 Wander Limits EEC Op2" "G.8262 Wander Transfer EEC Opt2" "G.8262 Wander Generation EEC Op1"\
        "G.8262 Wander Generation EEC Op2" "G.8272 PRTC" }
    }
    
    destructor {      
    }

    
    method createResult { } {
        
        if { $filterType == "MTIELF" } {
            set measurementAnalysis "DTEMTIE"
        } else {
            set measurementAnalysis "DTETDEV"
        }   
        next $measurementAnalysis
    }
    
    
    method setFilterType { type } {
        
        my selectSlot $timeMeasurement
        set filterType $type
        if { $filterType == "MTIELF" } {
            my enableMeasurement "DTEMTIE"
        } elseif { $filterType == "TDEVLF" } {
            my enableMeasurement "DTETDEV"
        } else {
            error " Error: Incorrect filter type: $type, Possible filter types are $filterTypesList "
        }
    }
    
    
    method getFilterType {} {
        return $filterType
    }
    
    
    method setMask { maskType } {
        
        my selectSlot $timeMeasurement
        if { $filterType == "MTIELF" } {
            set type "DTEMTIE"
        } else {
            set type "DTETDEV" 
        } 
        
        if { ($filterType == "MTIELF" &&  [lsearch $masksListMTIE $maskType] >= 0 ) || ($filterType == "TDEVLF" && [lsearch $masksListTDEV $maskType] >= 0 ) } {
            $paragonObject paragonset Cat $type Mask $maskType
            #$paragonObject calculateRequiredSetTrue
        } else {
            error " Error: Incorrect Mask type: $maskType for $filterType"
        }
        
    }
    

    method getMask {} {
        
        my selectSlot $timeMeasurement
        if { $filterType == "MTIELF" } {
                set type "DTEMTIE"
        } else {
                set type "DTETDEV" 
        }
        
        if { [$paragonObject paragonget Cat $type Mask] == "" } {
            return "no mask "
        } else {
            return [$paragonObject paragonget Cat $type Mask]
        }
    }  
}