################################################################################
#                                                                              #
#   paragon.tcl                                                                #
#                                                                              #
#   TCL interface to Calnex Paragon                                            #
#                                                                              #
#   Copyright (c) Calnex Solutions Ltd 2008 - 2011                             #
#                                                                              # 
#   The contents of this file should not be copied or distributed              #
#   without permission being granted by Calnex Solutions Ltd.                  #
#                                                                              #
#   All rights reserved.                                                       #
#                                                                              #
################################################################################

# The provided commands are described below. Errors are signalled using the
# TCL error method.

# Interface functions follow below - do not modify these 


################################################################################
#
#  Socket handling functions
#
################################################################################

#set [namespace current]::__lastError ""

proc logInfo { msg } {
    puts $msg
}


# ======================================================================================================================================

oo::class create ParagonX {
 variable paragonSocket
 variable calculateRequired
 variable idn
 variable serialNumber
 variable version
 variable type
 variable measurementConfiguration
 
    constructor {} {
        set calculateRequired False 
        variable connected 0
        Socket create paragonSocket
		
		set idn "Not Available"
		set serialNumber "Not Available"
		set version "Not Available"
		set type "Not Available"
    }
    
    destructor {
        variable paragonSocket
        $paragonSocket destroy
    }

method setMeasurementConfiguration {meas} {
	set measurementConfiguration $meas
}
method getMeasurementConfiguration {} {return $measurementConfiguration}

###########################################################################################################
variable dictionaryMeasurements

method addMeasurement { name measurementAnalysis1 timeMeasurement1 { filter "" } } {


    my selectSlot $timeMeasurement1

        
    if { $measurementAnalysis1 == "TE" } {
        set meas [ TEMeasurementAnalysis new [self] $timeMeasurement1 ]
        
    } elseif { $measurementAnalysis1 == "ATE" } {
        
        set meas [ ATEMeasurementAnalysis new [self] $timeMeasurement1 ]
        
    } elseif { $measurementAnalysis1 == "DTEMTIELF_TDEVLF" } {
        
        set filterTypesList { MTIELF TDEVLF }
        
        set meas [ DTEMTIELF_TDEVLFMeasurementAnalysis new [self] $timeMeasurement1 ]
        if { $filter == "MTIELF" || $filter == "TDEVLF" } {
            $meas setFilterType $filter
        } elseif { $filter == "" } {
            error " Error: Filter type was not defined, possible filter types are $filterTypesList " 
        } else {
            error " Error: Incorrect filter name: $filter, possible filter types are $filterTypesList "  
        }
        
    } elseif { $measurementAnalysis1 == "DTELF_HF" } {
        set filterTypesList { LF HF }
        set meas [ DTELF_HFMeasurementAnalysis new [self] $timeMeasurement1 ]
        if {  $filter == "LF" || $filter == "HF"} {
            $meas setFilterType $filter
        } elseif { $filter == "" } {
            error " Error: Filter type was not defined, possible filter types are $filterTypesList " 
        } else {
            error " Error: Incorrect filter name: $filter, possible filter types are $filterTypesList "  
        }      
        
    } elseif { $measurementAnalysis1 == "TETC" } {
        
        set meas [ TETCMeasurementAnalysis new [self] $timeMeasurement1 ]
        $meas setMeasurementAnalysis $filter
        
    } else {
        error " Error: Incorrect measurement analysis name: $measurementAnalysis1 "            
    }
    
    dict lappend dictionaryMeasurements $name $meas
    return $meas
}

method getMeasurement { name } {

    return [dict get $dictionaryMeasurements $name]   
}

method getDictionaryMeasurements { } {
    
    return $dictionaryMeasurements
}

method clearDictionaryMeasurements { } {
    set dictionaryMeasurements ""
}

method selectSlot { timeMeasurement } {
    [self] paragonset Cat SelectSlot $timeMeasurement
    [self] paragonset Cat EnabledSlot True
}
###################################################################################################

method calculate { } {
    if { $calculateRequired == True } {
        [self] paragonset Cat Calculate
        [self] waitforcat
        set calculateRequired False
    } 
}

method calculateRequiredSetTrue { } {
    set calculateRequired True
}


method calculateSpecified { timeMeasurement measurementAnalysis } {
    
    set measurementAnalysisBCList { "TE" "ATE" "DTE" "DTEHF" "DTEMTIE" "DTETDEV" }
    set measurementAnalysisTCList {  "FWD_CF_ACCURACY" "FWD_LATENCY" "FWD_CF_DELTA" "REV_CF_ACCURACY" "REV_LATENCY" "REV_CF_DELTA" "TWOWAY_CF_ACCURACY" }
    
    if { $calculateRequired == True } {
        
        if { [lsearch $measurementAnalysisBCList $measurementAnalysis] >= 0} {
            [self] paragonset Cat TE AllEnable False
            [self] paragonset Cat ATE AllEnable False
            [self] paragonset Cat DTE AllEnable False
            [self] paragonset Cat DTEHF AllEnable False
            [self] paragonset Cat DTEMTIE AllEnable False
            [self] paragonset Cat DTETDEV AllEnable False
            
        } elseif { [lsearch $measurementAnalysisTCList $measurementAnalysis] >= 0 } {
            [self] paragonset Cat FWD_CF_ACCURACY AllEnable False
            [self] paragonset Cat FWD_LATENCY AllEnable False
            [self] paragonset Cat FWD_CF_DELTA AllEnable False
            [self] paragonset Cat REV_CF_ACCURACY AllEnable False
            [self] paragonset Cat REV_LATENCY AllEnable False
            [self] paragonset Cat REV_CF_DELTA AllEnable False
            [self] paragonset Cat TWOWAY_CF_ACCURACY AllEnable False
        }
        
        [self] paragonset Cat SelectSlot $timeMeasurement
        [self] paragonset Cat EnabledSlot True
        [self] paragonset Cat $measurementAnalysis Enable True
        [self] paragonset Cat Calculate
        [self] waitforcat
        set calculateRequired False
        
    }
}

###################################################################################################

method getConnection {} {
    variable connected
	puts $connected
}

# Get the instrument type from the Idn string
method getInstrumentDetails {} {
	#set idnString [paragonget Idn]
	#set idnParts [split $idnString ,]
	#set type [lindex $idnParts 1]
	#return $type
	set idn [split [my paragonget Idn] ","]
	set serialNumber [lindex $idn 2]
	set type [lindex $idn 1]
	set version [lindex $idn 3]
	set version [split $version "="]
	set version [lindex $version 1]
	#logInfo $version
	logInfo "$idn\n"
}

method getInstrumentSerialNumber {} {
	return $serialNumber
}
method getInstrumentType {} {
	return $type
}
method getInstrumentVersion {} {
	return $version
}

method connect { instrumentIP {guihost localhost} {instrumentPort 9990} {guiPort 9000} {ll 1} } {

    variable connected
    variable paragonSocket
    set __lastError [paragonSocket getLastErrorValue]
    
    if {$connected} {
        set __lastError "ERROR: Already connected"
        my __chkForError "connect"
    }
    
    #puts "Before guiconnect: [paragonSocket getSockValue]"
    my guiconnect $guihost $guiPort
    #puts "After guiconnect: [paragonSocket getSockValue]"

    set status [ catch {
                     my remotecontrolon $ll
                     set connStatus [my isconnected]
                     if { $connStatus < 2 } {
                         set connThis [my isconnected $instrumentIP]
                         if { $connStatus == 1 && $connThis == 0 } {
                             set currIp [my paragonget RmtIpAddress]
                             set __lastError "ERROR: another instrument ($currIp) is already connected"
                             my __chkForError "connect"
                         }
                         if { $connThis == 0 } {
                             my instrumentconnect $instrumentIP $instrumentPort
                         }
                     }
                 }
               ]
    #puts "Status: $status"
    # Force Disconnect
    if { $status != 0 } {
        set err $__lastError
        my disconnect
        set __lastError $err
        my __chkForError "connect"
    } else {
        logInfo "connected to instrument"
        set connected 1
		my getInstrumentDetails
    }


}
method disconnect {} {
    variable connected
    variable paragonSocket
    
    catch { my instrumentdisconnect }
    catch { my remotecontroloff }
    catch { my guidisconnect }
    #set __sock ""
    paragonSocket setSockValue ""
    set connected 0

}

###### Rst processing ----------------------------------------------------------------
# The following has been copied out of paragon.tcl. This make Rst more robust

##
# Check to see if any wander generation is running
# If so, stop it
method _stopWander {} {
	#puts "stopWander"
	set isFreqOffsetRunning False
	set tolSingleState  	IDLE
	set tolTableState  		IDLE
	set tolMtieTdevState  	IDLE
	set synceTransientState IDLE
	set transferSingleState IDLE
	set transferTableState 	IDLE
	set transferTdevState 	IDLE
	
	catch {set isFreqOffsetRunning [paragonget WanderGeneration FrequencyOffset Enable]}
	catch {set tolSingleState  [paragonget WanderGeneration Tolerance Single State]}
	catch {set tolTableState  [paragonget WanderGeneration Tolerance Table State]}
	catch {set tolMtieTdevState  [paragonget WanderGeneration Tolerance MtieTdev State]}
	catch {set synceTransientState [paragonget WanderGeneration SyncETransient State]}
	catch {set transferSingleState [paragonget WanderGeneration Transfer Single State]}
	catch {set transferTableState [paragonget WanderGeneration Transfer Table State]}
	catch {set transferTdevState [paragonget WanderGeneration Transfer TDEV State]}
	
	if {$isFreqOffsetRunning} {
		paragonset WanderGeneration FrequencyOffset Enable FALSE
	}
	if {$tolSingleState != "IDLE"} {
		paragonset WanderGeneration Tolerance Single Enable FORCESTOP
	}
	if {$tolTableState != "IDLE"} {
		paragonset WanderGeneration Tolerance Table Enable FORCESTOP
	}
	if {$tolMtieTdevState != "IDLE"} {
		paragonset WanderGeneration Tolerance MtieTdev Enable STOP
	}
	if {$synceTransientState != "IDLE"} {
		paragonset WanderGeneration SyncETransient Enable STOP
	}
	if {$transferSingleState != "IDLE"} {
		catch {paragonset WanderGeneration Transfer Single CalibrateEnable FORCESTOP}
		catch {paragonset WanderGeneration Transfer Single GenerateEnable FORCESTOP}
	}
	if {$transferTableState != "IDLE"} {
		catch {paragonset WanderGeneration Transfer Table CalibrateEnable FORCESTOP}
		catch {paragonset WanderGeneration Transfer Table GenerateEnable FORCESTOP}
	}
	if {$transferTdevState != "IDLE"} {
		paragonset WanderGeneration Transfer TDEV Enable STOP
	}

}

method _stopPacketGeneration {} {
	#puts "stopPacketGeneration"
	set isPort0Running False
	set isPort1Running False
	
	catch {set isPort0Running [paragonget PacketGeneration #0 Enable]}
	catch {set isPort1Running [paragonget PacketGeneration #1 Enable]}

	if {$isPort0Running} {
		paragonset PacketGeneration #0 Enable FALSE
	}
	if {$isPort1Running} {
		paragonset PacketGeneration #1 Enable FALSE
	}
}

# Check to see if any jitter generation is running
# If so, stop it
method _stopJitter {} {
	#puts "stopJitter"
	set tolSingleState  	IDLE
	set tolTableState  		IDLE
	set maxTolTableState  	IDLE
	
	catch {set tolSingleState  [paragonget Jitter Tolerance Single State]}
	catch {set tolTableState  [paragonget Jitter Tolerance Table State]}
	catch {set maxTolTableState  [paragonget Jitter MaxTolerable Table State]}
	
	if {$tolSingleState != "IDLE"} {
		paragonset Jitter Tolerance Single Enable FORCESTOP
	}
	if {$tolTableState != "IDLE"} {
		paragonset Jitter Tolerance Table Enable FORCESTOP
	}
	if {$maxTolTableState != "IDLE"} {
		paragonset Jitter MaxTolerable Table Enable FORCESTOP
	}
}

method _stopMSE {} {
	#puts "stopMSE"
	set isMaster0Running False
	set isMaster1Running False
	set isSlaveRunning 	 False
	
	catch {set isMaster0Running [paragonget MasterSlave Master #0 Enabled]}
	catch {set isMaster1Running [paragonget MasterSlave Master #1 Enabled]}
	catch {set isSlaveRunning [paragonget MasterSlave Slave Enabled]}
	#puts "stopMSE: $isMaster0Running $isMaster1Running $isSlaveRunning"
	
	if {$isMaster0Running} {
		paragonset MasterSlave Master #0 Enabled FALSE
	}
	if {$isMaster1Running} {
		# Looks like the get doesn't return the correct status so ignore errors
		catch {paragonset MasterSlave Master #1 Enabled FALSE}
	}
	if {$isSlaveRunning} {
		catch {paragonset MasterSlave Slave Enabled FALSE}
	}
}

method _stopCapture {} {
	#puts "stopCapture"
	#set retry 0
	my stopcapture
	#while {[paragonget InstrumentStatus Capture IsRunning] == TRUE && $retry < 6} {
	#	after 500
	#	stopcapture
	#	incr retry
	#}
}

method _stopImpairment {} {
	#puts "stopImpairment"
	catch {my stopimpairment}
}

## Before executing Rst, make sure that the instrument is idle
# •	If a capture is in progress then we not idle.
# •	If Thru mode is active we are not idle.
# •	If import-export is active we are not idle.
#		We can't check for this and so we ignore it - unlikely to happen
# •	If MSE is active we are not idle.
# •	If impairments are running we are not idle.
# •	If wander generation is active we are not idle.
# •	If Jitter generation is active we are not idle.
# •	If Packet Generation is active we are not idle.
method _doResetPreparation {} {
	#puts "doResetPreparation"
	
	# Stop anything that is running
	my _stopCapture
	my _stopImpairment
	my _stopWander
	my _stopPacketGeneration
	my _stopJitter
	my _stopMSE
	
	#paragonset SimulMeasImpairMode MEASUREONLY
	#paragonset OperatingMode SERVICES
	#paragonset TxRxMode TRUE	
}


method _rstTranslation {arguments} {
	set args [join $arguments]

	# We want bypass set to false.
	# Once we are done with the other reset actions, we want to execute the Rst normally
	set bypass False

	if {[lindex $args 0 ] == "Rst"} {
		#puts "rstTranslation $args"
		my _doResetPreparation
	}
	return $bypass
}
###### End Rst procesing -------------------------------------------------------------


method paragonset { args } {
	set isBypass False
    set wireCmd "paragon.CSettingParameter(\"__SET__"

	set isBypass [my _rstTranslation $args]

    foreach arg $args {
        # Indexed: 1st char is '#'
        if {[string first \# $arg] == 0} {
            set idx [string range $arg 1 end]
            set arg "__IDX____#__"
            append arg $idx
        }
        append wireCmd "__#__" $arg
    }
    append wireCmd "\")"

# ifdef DJWAT_DEBUG
    #XXXDebugTrace "$wireCmd"
    
    paragonSocket send $wireCmd
    my __chkForError "paragonset $args"
}



method paragonget { args } {
    set wireCmd "paragon.CSettingParameter(\"__GET__"

    foreach arg $args {
        # Indexed: 1st char is '#'
        if {[string first \# $arg] == 0} {
            set idx [string range $arg 1 end]
            set arg "__IDX____#__"
            append arg $idx
        }
        append wireCmd "__#__" $arg
    }
    append wireCmd "\")"
    
    paragonSocket send $wireCmd
    my __chkForError "paragonget $args"


    set resp [ paragonSocket send "print(paragon.CSettingParameter_resp())" ]
    my __chkForError "paragonget $args"

    # Ignore the type ID character at the start of the response. Just
    # return the data as a string

    set resp [string range $resp 1 end]
    return $resp
}



method guiconnect { {hostname localhost} {port 9000} } {
    variable paragonSocket
    #global ::__sock


    logInfo "connecting to GUI $hostname:$port"
    # Open socket to the given port on the given hostname
    if [catch {
        #set __sock [socket $hostname $port]
        #puts [socket $hostname $port]
        paragonSocket setSockValue [socket $hostname $port]
        #paragonSocket connectSocket hostname port
        #fconfigure $__sock -buffering line
        fconfigure [paragonSocket getSockValue] -buffering line
    } ] {
        return -code error "Unable to connect to the GUI"
    }
    #if [gets $__sock line] != -1  
    if { [gets [paragonSocket getSockValue] line] != -1 } {
        logInfo "connected to GUI"
    } else {
        error "Error connecting to GUI "
    } 
}

method guidisconnect {} {
variable paragonSocket
#global ::__sock

# If we are trying to disconnect and aren't connected then simply ignore the GUI connection status
#    if { [ string compare "" $__sock ] == 0 } {
#        error "Not connected to GUI"
#    }
#    close $__sock
#    set __sock ""
    #if  [ string compare "" $__sock ] != 0  
    if { [ string compare "" [paragonSocket getSockValue] ] != 0 } {    
        #close $__sock
        close [paragonSocket getSockValue]
        #set __sock ""
        paragonSocket setSockValue ""
    }
}

method isconnected { {instrumentIP ""} } {
    variable paragonSocket
    #global ::__sock
# Send generates an error if not connected to the GUI. For isconnected, we should simply return false
    if { [my isguiconnected] == 0 } {
        set resp 0
    } else {
        set resp [ paragonSocket send "paragon.remoteprint(paragon.isconnected(\"$instrumentIP\"))" ]
        my __chkForError "isconnected"
    }
    return $resp
}

method isguiconnected {} {
    variable paragonSocket
    #global ::__sock
    #if  [ string compare "" $__sock ] == 0 
    if { [ string compare "" [paragonSocket getSockValue] ] == 0 } {
        set resp 0
    } else {
        set resp 1
    }
    return $resp
}

method remotecontrolon { {ll 1} } {
    variable paragonSocket
    paragonSocket send "paragon.on($ll)"
    my __chkForError "remotecontrolon"
}

method remotecontroloff {} {
    variable paragonSocket
    paragonSocket send "paragon.off()"
    my __chkForError "remotecontroloff"
}

method instrumentconnect { ipAddress {port 9990} } {
    variable paragonSocket
    logInfo "connecting to instrument $ipAddress:$port"
    paragonSocket send "paragon.connect(\"$ipAddress\",\"$port\")"
    my __chkForError "instrumentconnect $ipAddress:$port"
}

method instrumentdisconnect {} {
    variable paragonSocket
    paragonSocket send "paragon.disconnect()"
    my __chkForError "instrumentdisconnect"
}

  # note that a change in "Wat Status" command causes a change in the implementation of tcl command  wait.
method waitforcat {} {
	set ok [ my paragonget "Wat" "Status"]
	set log 0
	while { $ok == 1 } {
		if {$log == 0} {
			set log 1
		}
        after 1000
		set ok [ my paragonget "Wat" "Status"]
	}
	#logInfo "Wait operation completed"
}


method waitForCat {} {
	[self] waitforcat
}


method setoperatingmode { operatingMode } {
    variable paragonSocket
    paragonSocket send "paragon.CSettingParameter(\"__SET____#__OperatingMode__#__$operatingMode\")"
    my __chkForError "setoperatingmode $operatingMode"
}

method startpacketcapture {} {
    variable paragonSocket
    paragonSocket send "paragon.startpacketcapture()"
    my __chkForError "startpacketcapture"
}

method starttimingcapture {} {
    variable paragonSocket
    paragonSocket send "paragon.starttimingcapture()"
    my __chkForError "starttimingcapture"
}

method stopcapture {} {
    variable paragonSocket
    paragonSocket send "paragon.stopcapture()"
    my __chkForError "stopcapture"
}

method startimpairment {} {
    variable paragonSocket
    paragonSocket send "paragon.startimpairment()"
    my __chkForError "startimpairment"
}

method stopimpairment {} {
    variable paragonSocket
    paragonSocket send "paragon.stopimpairment()"
    my __chkForError "stopimpairment"
}


method starttodcapture {} {
    variable paragonSocket
    paragonSocket send "paragon.starttodcapture()"
    my __chkForError "starttodcapture"
}

method stoptodcapture {} {
    variable paragonSocket
    paragonSocket send "paragon.stoptodcapture()"
    my __chkForError "stoptodcapture"
}

# These use the Lua string functions to convert to Windows-style filename separators

method exportdata { filename } {
    variable paragonSocket
    paragonSocket send "paragon.exportdata(string.gsub(\"$filename\",\"/\",\"\\\\\"))"
    my __chkForError "exportdata  $filename"
}

method importdata { filename } {
    variable paragonSocket
    paragonSocket send "paragon.importdata(string.gsub(\"$filename\",\"/\",\"\\\\\"))"
    my __chkForError "importdata $filename"
}

method store { filename } {
    variable paragonSocket
    paragonSocket send "saveSettings(string.gsub(\"$filename\",\"/\",\"\\\\\"))"
    my __chkForError "store $filename"
}

method recall { filename } {
    variable paragonSocket
    paragonSocket send "loadSettings(string.gsub(\"$filename\",\"/\",\"\\\\\"))"
    my __chkForError "recall $filename"
}

# These use the Lua string functions to convert to Windows-style filename separators

method importimpairmentdata { port filename } {
    variable paragonSocket
    paragonSocket send "paragon.importimpairmentdata(\"$port\",string.gsub(\"$filename\",\"/\",\"\\\\\"))"
    my __chkForError "importimpairmentdata $port $filename"
}

method setsonetowsts1trib { trib } {
    variable paragonSocket
    paragonSocket send "paragon.CSettingParameter(\"__SET____#___SonetSts1Trib__#__$trib\")"
    my __chkForError "setsonetowsts1trib $trib"
}

method setsonetowvttrib { trib } {
    variable paragonSocket
    paragonSocket send "paragon.CSettingParameter(\"__SET____#___SonetVtTrib__#__$trib\")"
    my __chkForError "setsonetowvttrib $trib"
}

method setsonetdelay { delay } {
    variable paragonSocket
    paragonSocket send "paragon.CSettingParameter(\"__SET____#___SonetDelay__#__$delay\")"
    my __chkForError "setsonetdelay $delay"
}


method __chkForError { command } {
	#global ::__lastError
    variable paragonSocket
    set __lastError [paragonSocket getLastErrorValue]
    if {[string compare $__lastError ""] != 0 } { error "$__lastError (command=\"$command\")" }
}


method _queryerrors {} {
    variable paragonSocket
    paragonSocket send "paragon._queryerrors()"
    my __chkForError "queryerrors"
}


}


#================================================================================================

# not finished Socket class

oo::class create Socket {
     constructor {} {
        variable __sock ""
        variable __lastError ""
    }
    
    method getSockValue {} {
        variable __sock
        return $__sock
    }
    
    method setSockValue { value } {
        variable __sock
        set __sock $value
    }
    
    method getLastErrorValue {} {
        variable __lastError
        return $__lastError
    }
    
    method setLastErrorValue { value } {
        variable __lastError
        set __lastError $value
    }
  

    
    method send { message } {
	variable __sock
	variable __lastError

    if { [ string compare "" $__sock ] == 0 } {
        error "Not connected to GUI"
    }
    
    # Send the message to the GUI
    puts $__sock $message
    # 
    # Wait for response from the GUI
    #     Errors are returned as a string beginning 'ERROR: '
    #     Ready is signalled by '__PARAGON__' prompt
    #     Anything else is considered a response
    # 
    set response ""
    set __lastError ""
    
    while { 1 } {
        if { [gets $__sock line] == -1 } {
            error "Socket error"
        }
        
        if {[string first "ERROR: " $line] == 0 } {
            set __lastError "$line"
        }
            
		if {[string first "WARNING: " $line] == 0 } {
            set __lastError "$line"
        }
        if {[string compare $line "__PARAGON__"] == 0 } { 
            break
        } else {
            set response $line
        } 
    }
    
    return $response 
}
    
    
}
